from autoscript_tem_microscope_client import TemMicroscopeClient
from autoscript_tem_microscope_client.structures import *

import time
import math

microscope = TemMicroscopeClient()
microscope.connect("192.168.0.1")

# Beam blanking check
print("Starting blank check")
if ( microscope.optics.is_beam_blanked ):
    print("Beam is blanked")
else:
    print("Beam is active. Set beam blanked.")
    microscope.optics.blank()

# EDS mapping conditions
dwell_time = 0.1 # in second
frame_num = 1 # frame number
image_size_x = 64 # in pixel
image_size_y = 64 # in pixel

# Timing conditions
Adjust_constant = -0.04425802137702703
eff_dwell_time = dwell_time + Adjust_constant # Actual dwell time (subtract input dwell time from processing time)
wait_time=0.4

# Rocking conditions
tilt_semi_angle = 40   # in mrad
tilt_rotation = 0  # in degree (positive value is clock wise)

tilt_semi_angle = tilt_semi_angle * 0.01234567 # transformation coefficient mrad-> actual input value

# Calculate start beam tilt value
initial_angle0 = microscope.optics.deflectors.beam_tilt
tilt_semi_angle_X = tilt_semi_angle * (math.cos(math.radians(-tilt_rotation))-math.sin(math.radians(-tilt_rotation)))
tilt_semi_angle_Y = tilt_semi_angle * (math.sin(math.radians(-tilt_rotation))+math.cos(math.radians(-tilt_rotation)))
initial_angle = initial_angle0 - Point(tilt_semi_angle_X, tilt_semi_angle_Y)

print("Waiting for unblank. Please start EDS map with Velox.")
temp = 0
while( microscope.optics.is_beam_blanked ):
    temp+=1

time.sleep(wait_time)
initial_time0 =time.time()
for jj in range(0,frame_num):
    microscope.optics.deflectors.beam_tilt = initial_angle
    for j in range(0, image_size_y):
        for i in range(0,image_size_x):
            tilt = initial_angle + Point( \
                (((tilt_semi_angle*2/ image_size_x) * i) * math.cos(math.radians(-tilt_rotation)) \
                 -((tilt_semi_angle*2/ image_size_y) * j) * math.sin(math.radians(-tilt_rotation))), \
                (((tilt_semi_angle*2/ image_size_y) * i) * math.sin(math.radians(-tilt_rotation)) \
                 +((tilt_semi_angle*2/ image_size_y) * j) * math.cos(math.radians(-tilt_rotation))) \
                )
            microscope.optics.deflectors.beam_tilt = tilt
            print(j,i)
            time.sleep(eff_dwell_time)

end_time0 = time.time()

# Reset beam tilt
microscope.optics.deflectors.beam_tilt = initial_angle0

time_diff = end_time0 - initial_time0
proc_time = time_diff/(image_size_x*image_size_y)
if eff_dwell_time-proc_time > 0:
    print("Dwell time is too short.", eff_dwell_time-proc_time)
else:
    print("Adjustment time (s): ", eff_dwell_time-proc_time)
